# 2019/05/14 Ver 1.0
# Do not this comentout

require 'json'
require 'time'
require 'rack/request'
require 'rack/response'

class VCOrderFileCreator
  # define constants
  FILE_PATH    = '/tmp/'   # order file output destination
  EC_ID        = '1000001' # Advertiser ID
  APPEND_ORDER = false     # true: append the data to order file, false: overwrite the data to order file
  TIMEZONE     = '+09:00'  # Asia/Tokyo


  def create_order_file_in_json( order, is_append = false, to_encoding = 'UTF-8' )
    if is_append
      mode = 'a'
      file_name = EC_ID + '_' + Time.new.utc.localtime(TIMEZONE).strftime('%Y%m%d') + '_P.json'
    else
      mode = 'w'
      file_name = order['ORDER_ID'].slice(0,248) + '_P.json'
    end

    file_path = FILE_PATH + file_name
    File.open( file_path, mode + ':' + to_encoding ) do |contents|
      JSON.dump( order, contents )
    end

    File.chmod( 0666, file_path )
  end

  def create_order_file( env, order_info )
    req = Rack::Request.new( env )

    cookie_key = '_vc_cec_' + EC_ID
    vc_cookie = req.cookies()[ cookie_key ]

    now = Time.new.utc.localtime( TIMEZONE )

    order = Hash.new
    order_info.each { |key, value| order[ key ] = value }

    unless order['FC'].nil? then
      fc = req.cookies()[ '_vc_citi_' + EC_ID ]
      order['FC'] = fc unless fc.nil?
    end

    order['EC_ID']                = EC_ID
    order['STATUS']               = 'P'
    order['SALES_DATE']           = now.strftime( '%Y/%m/%d' )
    order['SALES_TIME']           = now.strftime( '%H:%M:%S' )
    order['REMOTE_ADDR']          = env['REMOTE_ADDR'].nil? ? '' : env['REMOTE_ADDR']
    order['HTTP_USER_AGENT']      = env['HTTP_USER_AGENT'].nil? ? '' : env['HTTP_USER_AGENT']
    order['HTTP_X_FORWARDED_FOR'] = env['HTTP_X_FORWARDED_FOR'].nil? ? '' : env['HTTP_X_FORWARDED_FOR']
    order['HTTP_VIA']             = env['HTTP_VIA'].nil? ? '' : env['HTTP_VIA']
    order['VC_COOKIE']            = vc_cookie.nil? ? '' : vc_cookie

    create_order_file_in_json( order, APPEND_ORDER )
  end

  def get_upcase_keys_hash_from_qstr(qstr)
    q_hash = Hash.new
    qstr.split('&').each do |query|
      key, val = query.split('=', 2)
      q_hash[key.upcase] = val
    end
    return q_hash
  end

   # for testing
#  def call(env)
#    order_info = Hash.new
#    order_info['ORDER_ID'   ] = '001'
#    order_info['SALES_NAME' ] = '1234567'
#    order_info['SALES_TEL'  ] = '*****'
#    order_info['SALES_EMAIL'] = 'gonbe'
#    order_info['CS'         ] = 'N'
#    order_info['UID'        ] = 'customer12345'
#
#    detail1 = Hash.new
#    detail1['ITEM_QTY'      ] = 1
#    detail1['ITEM_PRICE'    ] = 100
#    detail1['ITID'          ] = 'it1'
#    detail1['PRODUCT_ID'    ] = 'product1'
#    detail1['ITEM_NAME'     ] = 'item1'
#    detail1['ITEM_PRGRID'   ] = 'prg1'
#    detail1['ITEM_DETAIL_ID'] = 'detail1'
#
#    detail2 = Hash.new
#    detail2['ITEM_QTY'      ] = 2
#    detail2['ITEM_PRICE'    ] = 300
#    detail2['ITEM_NAME'     ] = 'item2'
#    detail2['ITEM_DETAIL_ID'] = 'detail2'
#
#    order_info['DETAIL'] = Array.new
#    order_info['DETAIL'].push( detail1 )
#    order_info['DETAIL'].push( detail2 )
#
#    create_order_file( env, order_info ) # Pass env and order_info to create_order_file.
#                                         # env is CGI-environment-vairables likes HTTP_COOKIE, HTTP_USER_AGENT, etc.
#
#    res = Rack::Response.new { |r|
#      r.status = 200
#      r['Content-Type'] = 'text/html;charset=utf-8'
#    }
#    res.finish
#  end

  # execute ruby in javascript
#  def call(env)
#    # using env and order_info to create_order_file.
#    # env is CGI-environment-vairables likes HTTP_COOKIE, HTTP_USER_AGENT, etc.
#    order_info = get_upcase_keys_hash_from_qstr(env['QUERY_STRING'])
#    create_order_file(env, order_info)
#
#    res = Rack::Response.new { |r|
#      r.status = 200
#      r['Content-Type'] = 'text/html;charset=utf-8'
#    }
#    res.finish
#  end
end

